@@ -1001,11 +1001,17 @@ class API(ModelView):
             related_model = get_related_model(self.model, relationname)
             relations = frozenset(get_relations(related_model))
             deep = dict((r, {}) for r in relations)
-            # for security purposes, don't transmit list as top-level JSON
-            if is_like_list(instance, relationname):
-                result = self._paginated(list(related_value), deep)
+            if relationinstid is not None:
+                related_value_instance = get_by(self.session, related_model, relationinstid)
+                if related_value_instance is None:
+                    abort(404)
+                result = to_dict(related_value_instance, deep)
             else:
-                result = to_dict(related_value, deep)
+                # for security purposes, don't transmit list as top-level JSON
+                if is_like_list(instance, relationname):
+                    result = self._paginated(list(related_value), deep)
+                else:
+                    result = to_dict(related_value, deep)
         for postprocessor in self.postprocessors['GET_SINGLE']:
             postprocessor(result=result)
         return jsonpify(result)
@@ -1134,16 +1140,18 @@ class API(ModelView):
             self.session.add(instance)
             self.session.commit()
             result = self._inst_to_dict(instance)
-
-            for postprocessor in self.postprocessors['POST']:
-                postprocessor(result=result)
+            primary_key = str(result[primary_key_name(instance)])
 
             # The URL at which a client can access the newly created instance
             # of the model.
-            primary_key = primary_key_name(instance)
-            url = '%s/%s' % (request.base_url, result[primary_key])
+
+            url = '%s/%s' % (request.base_url, primary_key)
             # Provide that URL in the Location header in the response.
             headers = dict(Location=url)
+
+            for postprocessor in self.postprocessors['POST']:
+                postprocessor(result=result)
+                        
             return jsonify_status_code(201, headers=headers, **result)
         except self.validation_exceptions, exception:
             return self._handle_validation_exception(exception)
